@@ -1,5 +1,7 @@
+import contextlib
 import copy
 import logging
+import time
 from typing import Dict, Union
 
 import serial as pyserial
@@ -43,40 +45,58 @@ class Serial(DuplicateStdoutMixin):
             self.port_config = copy.deepcopy(self.DEFAULT_PORT_CONFIG)
             self.port_config.update(**kwargs)
             self.proc = pyserial.serial_for_url(self.port, **self.port_config)
-        elif isinstance(port, pyserial.Serial):  # pyserial instance
+        else:  # pyserial instance
             self.proc = port
             self.proc.timeout = self.DEFAULT_PORT_CONFIG['timeout']  # set read timeout
             self.port = self.proc.port
-        else:
-            raise ValueError('Port should be a string or a pyserial.Serial instance')
 
         self.pexpect_proc = pexpect_proc
-        self.occupied_ports[self.port] = None
-        logging.debug(f'occupied {self.port}')
+
+        self._post_init()
 
         self._start()
 
+        self._finalize_init()
+
+    def _post_init(self):
+        pass
+
     def _start(self):
         pass
 
+    def _finalize_init(self):
+        self.occupied_ports[self.port] = None
+        logging.debug(f'occupied {self.port}')
+
     def _forward_io(self, pexpect_proc: PexpectProcess) -> None:
-        rest = b''
         while self.proc.is_open:
             try:
-                s = self.proc.read_until()
-                s = rest + s
-
-                res = s.rsplit(b'\n', maxsplit=1)
-                if len(res) == 1:
-                    rest = s
-                else:
-                    s = res[0]
-                    rest = res[1]
-                    if s:
-                        pexpect_proc.write(s + b'\n')
+                s = self.proc.read_all()
+                pexpect_proc.write(s)
             except:  # noqa daemon thread may run at any case
-                try:
-                    pexpect_proc.write(rest)  # try to write the rest of line
-                except:  # noqa daemon thread may run at any case
-                    pass
                 break
+
+    def stop_redirect_thread(self) -> bool:
+        """
+        Close the serial port and reopen it to kill the redirect daemon thread.
+
+        Returns:
+            Killed the redirect thread or not
+        """
+        killed = False
+        if self._forward_io_thread and self._forward_io_thread.is_alive():
+            self.proc.close()
+            time.sleep(0.1)
+            self.proc.open()  # to kill the redirect stdout thread
+            killed = True
+
+        return killed
+
+    @contextlib.contextmanager
+    def disable_redirect_thread(self):
+        killed = self.stop_redirect_thread()
+
+        yield killed
+
+        if killed:
+            self.create_forward_io_thread(self.pexpect_proc)
